@@ -1,10 +1,9 @@
 package Net::DNS::ZoneFile;
 
 #
-# $Id: ZoneFile.pm 1510 2016-10-11 12:29:10Z willem $
+# $Id: ZoneFile.pm 1709 2018-09-07 08:03:09Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1510 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1709 $)[1];
 
 
 =head1 NAME
@@ -44,9 +43,13 @@ automatically to all subsequent records.
 
 
 use strict;
+use warnings;
 use integer;
 use Carp;
-use FileHandle;
+use IO::File;
+
+use base qw(Exporter);
+our @EXPORT = qw(parse read readfh);
 
 use constant PERLIO => defined eval 'require PerlIO';
 
@@ -61,7 +64,7 @@ require Net::DNS::RR;
 
     $zonefile = new Net::DNS::ZoneFile( 'filename', ['example.com'] );
 
-    $handle   = new FileHandle( 'filename', '<:encoding(ISO8859-7)' );
+    $handle   = new IO::File( 'filename', '<:encoding(ISO8859-7)' );
     $zonefile = new Net::DNS::ZoneFile( $handle, ['example.com'] );
 
 The new() constructor returns a Net::DNS::ZoneFile object which
@@ -72,7 +75,7 @@ exhausted or all references to the ZoneFile object cease to exist.
 
 The optional second argument specifies $ORIGIN for the zone file.
 
-Character encoding is specified indirectly by creating a FileHandle
+Character encoding is specified indirectly by creating a file handle
 with the desired encoding layer, which is then passed as an argument
 to new(). The specified encoding is propagated to files introduced
 by $include directives.
@@ -85,14 +88,14 @@ sub new {
 	$self->_origin(shift);
 
 	if ( ref($file) ) {
-		$self->{filename} = $self->{handle} = $file;
+		$self->{filename} = $self->{filehandle} = $file;
 		$self->{fileopen} = {};
-		return $self if ref($file) =~ /FileHandle|IO::File|GLOB|Text/;
+		return $self if ref($file) =~ /IO::File|FileHandle|GLOB|Text/;
 		croak 'argument not a file handle';
 	}
 
 	$self->{filename} = $file ||= '';
-	$self->{handle} = new FileHandle($file) or croak qq($! "$file");
+	$self->{filehandle} = new IO::File($file) or croak "$! $file";
 	$self->{fileopen}{$file}++;
 	return $self;
 }
@@ -166,7 +169,7 @@ Returns the number of the last line read from the current zone file.
 sub line {
 	my $self = shift;
 	return $self->{eom} if defined $self->{eom};
-	return $self->{handle}->input_line_number;
+	return $self->{filehandle}->input_line_number;
 }
 
 
@@ -203,15 +206,18 @@ sub ttl {
 Applications which depended on the defunct Net::DNS::ZoneFile 1.04
 CPAN distribution will continue to operate with minimal change using
 the compatibility interface described below.
+New application code should use the object-oriented interface.
 
     use Net::DNS::ZoneFile;
 
     $listref = Net::DNS::ZoneFile->read( $filename );
     $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
-    $listref = Net::DNS::ZoneFile->readfh( $handle, $include_dir );
+    $listref = Net::DNS::ZoneFile->readfh( $filehandle );
+    $listref = Net::DNS::ZoneFile->readfh( $filehandle, $include_dir );
 
     $listref = Net::DNS::ZoneFile->parse(  $string );
+    $listref = Net::DNS::ZoneFile->parse( \$string );
     $listref = Net::DNS::ZoneFile->parse(  $string, $include_dir );
     $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
 
@@ -225,22 +231,21 @@ be obtained directly by calling any of these methods in list context.
 
     @rr = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
+The partial result is returned if an error is encountered by the parser.
+
 
 =head2 read
 
+    $listref = Net::DNS::ZoneFile->read( $filename );
     $listref = Net::DNS::ZoneFile->read( $filename, $include_dir );
-    @rr = Net::DNS::ZoneFile->read( $filename, $include_dir );
 
-read() parses the specified zone file and returns a reference to the
-list of Net::DNS::RR objects representing the RRs in the file.
-The return value is undefined if the zone data can not be parsed.
-
-When called in list context, the partial result is returned if an
-error is encountered by the parser.
+read() parses the contents of the specified file
+and returns a reference to the list of Net::DNS::RR objects.
+The return value is undefined if an error is encountered by the parser.
 
 =cut
 
-use vars qw($include_dir);		## dynamically scoped
+our $include_dir;			## dynamically scoped
 
 sub _filename {				## rebase unqualified filename
 	my $name = shift;
@@ -248,7 +253,7 @@ sub _filename {				## rebase unqualified filename
 	return $name unless $include_dir;
 	require File::Spec;
 	return $name if File::Spec->file_name_is_absolute($name);
-	return $name if -f $name;
+	return $name if -f $name;	## file in current directory
 	return File::Spec->catfile( $include_dir, $name );
 }
 
@@ -278,20 +283,11 @@ sub _read {
 
 	use overload ( '<>' => 'readline' );
 
-	use constant OVERLOAD_OK => scalar eval 'no integer; $] > 5.006';
-
 	sub new {
 		my $self = bless {}, shift;
 		my $data = shift;
 		$self->{data} = [split /\n/, ref($data) ? $$data : $data];
-
-		return OVERLOAD_OK ? $self : do {		# Plan B
-			require IO::File;
-			my $fh = IO::File->new_tmpfile() or die "$!";
-			while ( my $line = $self->readline ) { print $fh $line, "\n"; }
-			seek $fh, 0, 0;
-			return $fh;
-		};
+		return $self;
 	}
 
 	sub readline {
@@ -314,11 +310,12 @@ sub _read {
 
 =head2 readfh
 
-    $listref = Net::DNS::ZoneFile->readfh( $handle, $include_dir );
+    $listref = Net::DNS::ZoneFile->readfh( $filehandle );
+    $listref = Net::DNS::ZoneFile->readfh( $filehandle, $include_dir );
 
-readfh() parses data from the specified file handle and returns a
-reference to the list of Net::DNS::RR objects representing the RRs
-in the file.
+readfh() parses data from the specified file handle
+and returns a reference to the list of Net::DNS::RR objects.
+The return value is undefined if an error is encountered by the parser.
 
 =cut
 
@@ -329,17 +326,21 @@ sub readfh {
 
 =head2 parse
 
+    $listref = Net::DNS::ZoneFile->parse(  $string );
+    $listref = Net::DNS::ZoneFile->parse( \$string );
     $listref = Net::DNS::ZoneFile->parse(  $string, $include_dir );
     $listref = Net::DNS::ZoneFile->parse( \$string, $include_dir );
 
-parse() interprets the zone file text in the argument string and
-returns a reference to the list of Net::DNS::RR objects representing
-the RRs.
+parse() interprets the text in the argument string
+and returns a reference to the list of Net::DNS::RR objects.
+The return value is undefined if an error is encountered by the parser.
 
 =cut
 
 sub parse {
-	my ($text) = reverse @_;
+	my ($arg1) = @_;
+	shift if !ref($arg1) && $arg1 eq __PACKAGE__;
+	my $text = shift;
 	return &readfh( new Net::DNS::ZoneFile::Text($text), @_ );
 }
 
@@ -353,8 +354,6 @@ sub parse {
 
 	use overload ( '<>' => 'readline' );
 
-	use constant OVERLOAD_OK => scalar eval 'no integer; $] > 5.006';
-
 	sub new {
 		my $self = bless {}, shift;
 		my ( $range, $template, $line ) = @_;
@@ -371,14 +370,7 @@ sub parse {
 		$self->{count} = int( ( $last - $first ) / $step ) + 1;
 
 		@{$self}{qw(instant step template line)} = ( $first, $step, $template, $line );
-
-		return OVERLOAD_OK ? $self : do {		# Plan B
-			require IO::File;
-			my $fh = IO::File->new_tmpfile() or die "$!";
-			while ( my $line = $self->readline ) { print $fh $line, "\n"; }
-			seek $fh, 0, 0;
-			return $fh;
-		};
+		return $self;
 	}
 
 	sub readline {
@@ -419,8 +411,8 @@ sub parse {
 		return substr sprintf( "%01.$digit$format", $value ), -$length if $format =~ /[doxX]/;
 
 		my $nibble = join( '.', split //, sprintf ".%32.32lx", $value );
-		return lc reverse substr $nibble, -$length if $format =~ /[n]/;
-		return uc reverse substr $nibble, -$length if $format =~ /[N]/;
+		return reverse lc( substr $nibble, -$length ) if $format =~ /[n]/;
+		return reverse uc( substr $nibble, -$length ) if $format =~ /[N]/;
 		die "unknown $format format";
 	}
 
@@ -434,7 +426,7 @@ sub _generate {				## expand $GENERATE into input stream
 
 	delete $self->{latest};					# forget previous owner
 	$self->{parent} = bless {%$self}, ref($self);		# save state, create link
-	$self->{handle} = $handle;
+	$self->{filehandle} = $handle;
 }
 
 
@@ -443,7 +435,7 @@ my $LEX_REGEX = q/("[^"]*"|"[^"]*$)|;[^\n]*|([()])|(^\s)|[ \t\n\r\f]/;
 sub _getline {				## get line from current source
 	my $self = shift;
 
-	my $fh = $self->{handle};
+	my $fh = $self->{filehandle};
 	while (<$fh>) {
 		next if /^\s*;/;				# discard comment line
 		next unless /\S/;				# discard blank line
@@ -541,15 +533,15 @@ sub _include {				## open $INCLUDE file
 	my $opened = {%{$self->{fileopen}}};
 	croak qq(recursive \$INCLUDE $file) if $opened->{$file}++;
 
-	my @discipline = PERLIO ? ( join ':', '<', PerlIO::get_layers $self->{handle} ) : ();
-	my $handle = new FileHandle( $file, @discipline ) or croak qq($! "$file");
+	my @discipline = PERLIO ? ( join ':', '<', PerlIO::get_layers $self->{filehandle} ) : ();
+	my $handle = new IO::File( $file, @discipline ) or croak "$! $file";
 
 	delete $self->{latest};					# forget previous owner
 	$self->{parent} = bless {%$self}, ref($self);		# save state, create link
-	$self->{context} = origin Net::DNS::Domain($root) if $root;
+	$self->{context}  = origin Net::DNS::Domain($root) if $root;
 	$self->{filename} = $file;
 	$self->{fileopen} = $opened;
-	return $self->{handle} = $handle;
+	return $self->{filehandle} = $handle;
 }
 
 
